@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 #include "../../gst/gst_private.h"
 
@@ -159,6 +159,7 @@ GST_START_TEST (test_class)
   fail_if (klass->elementfactory == NULL);
   fail_if (klass->elementfactory != factory);
 
+  gst_object_unref (factory);
 }
 
 GST_END_TEST;
